/* ------------------------------------------------------------------------
   (C) Copyright Marxmeier Software AG 2009-2014

   Project:  Eloquence JDLG Plugin Interface
   File:     Bar.java
   Function: JDLG Charts Plugin: Bar Chart
   $Id: Bar.java,v 1.8 2014/12/11 12:17:23 rhg Exp $

   Edit history:
   2009-04-06, ap   - creation
   2009-04-28, mike - finalized
   2014-04-08, rhg  - api version 1.1 additions
   2014-12-03, rhg  - revised, enhancements, bugfixes, code cleanup
 ------------------------------------------------------------------------ */

package com.eloquence.plugin.charts;

import com.eloquence.api.*;

import java.awt.*;
import java.util.*;

import org.jfree.chart.*;
import org.jfree.chart.plot.*;
import org.jfree.chart.renderer.category.*;
import org.jfree.chart.labels.*;
import org.jfree.ui.*;

public class Bar
   extends Axis
{
   // ------------------------------------------------------------------------
   // Plugin
   // ------------------------------------------------------------------------

   public Component onCreate(Jdlg jdlg)
   {
      if (!jdlg.requiresApiVersion(1, 0))
         return null;

      // label: different default value
      showLabels = false;

      peerPanel = new ChartPanel(null);
      return peerPanel;
   }

   public void onEloq2Swing()
   {
      if (chartModified)
      {
         // createBarChart(java.lang.String title,
         //               java.lang.String categoryAxisLabel,
         //               java.lang.String valueAxisLabel,
         //               CategoryDataset dataset,
         //               PlotOrientation orientation,
         //               boolean legend,
         //               boolean tooltips,
         //               boolean urls)

         String titleLabel = title.length() > 0 ? title : null;
         final String categoryLabel = "Category";
         final String valueLabel = "Value";
         if (paint3d)
         {
            if (stacked)
               peerPanel.setChart(
                  ChartFactory.createStackedBarChart3D(
                     titleLabel, categoryLabel, valueLabel,
                     dataset, PlotOrientation.VERTICAL,
                     legend, true, false));
            else
               peerPanel.setChart(
                  ChartFactory.createBarChart3D(
                     titleLabel, categoryLabel, valueLabel,
                     dataset, PlotOrientation.VERTICAL,
                     legend, true, false));
         }
         else {
            if (stacked)
               peerPanel.setChart(
                  ChartFactory.createStackedBarChart(
                     titleLabel, categoryLabel, valueLabel,
                     dataset, PlotOrientation.VERTICAL,
                     legend, true, false));
            else
               peerPanel.setChart(
                  ChartFactory.createBarChart(
                     titleLabel, categoryLabel, valueLabel,
                     dataset, PlotOrientation.VERTICAL,
                     legend, true, false));
         }

         chartModified = false;
         attributeModified = true;
         datasetModified = true;
      }

      super.onEloq2Swing();

      JFreeChart chart = null;
      CategoryPlot plot = null;
      BarRenderer renderer = null;

      if (datasetModified)
      {
         if (chart == null)
         {
            chart = peerPanel.getChart();
            plot = (CategoryPlot)chart.getPlot();
            renderer = (BarRenderer)plot.getRenderer();
         }

         int numRows = rows.size();
         for (int rowIndex = 0; rowIndex < numRows; rowIndex++)
         {
            Row row = (Row)rows.get(rowIndex);
            renderer.setSeriesPaint(rowIndex, row.getColor());
         }
      }

      if (attributeModified)
      {
         if (chart == null)
         {
            chart = peerPanel.getChart();
            plot = (CategoryPlot)chart.getPlot();
            renderer = (BarRenderer)plot.getRenderer();
         }

         chart.setBorderVisible(border);
         chart.setPadding(borderInsets);
         plot.setBackgroundPaint(chartBgc);
         plot.setForegroundAlpha((float)fgAlpha / 100);
         renderer.setDrawBarOutline(outline);
         renderer.setShadowVisible(shadow);
         renderer.setItemMargin((double)margin / 100);
      }

      if (datasetModified || attributeModified)
      {
         StandardCategoryItemLabelGenerator labelGenerator;
         if (showLabels)
            labelGenerator = new StandardCategoryItemLabelGenerator(
               getLabelFormat(), valueFormat(), percentFormat());
         else
            labelGenerator = null;

         int numRows = rows.size();
         for (int rowIndex = 0; rowIndex < numRows; rowIndex++)
         {
            renderer.setSeriesItemLabelGenerator(rowIndex, labelGenerator);
            renderer.setSeriesItemLabelsVisible(rowIndex, showLabels);
         }
      }

      datasetModified = false;
      attributeModified = false;
   }

   // ------------------------------------------------------------------------
   // get/set paint3d
   // ------------------------------------------------------------------------

   private static final String PAINT3D_ATTR_NAME = "paint3d";
   private boolean paint3d = false;

   private final void setPaint3d(int paint3d)
   {
      this.paint3d = paint3d != 0;
      chartModified = true;
   }

   private final int getPaint3d()
   {
      return paint3d ? 1 : 0;
   }

   // ------------------------------------------------------------------------
   // get/set outline
   // ------------------------------------------------------------------------

   private static final String OUTLINE_ATTR_NAME = "outline";
   private boolean outline = false;

   private final void setOutline(int outline)
   {
      this.outline = outline != 0;
      chartModified = true;
   }

   private final int getOutline()
   {
      return outline ? 1 : 0;
   }

   // ------------------------------------------------------------------------
   // get/set shadow
   // ------------------------------------------------------------------------

   private static final String SHADOW_ATTR_NAME = "shadow";
   private boolean shadow = false;

   private final void setShadow(int shadow)
   {
      this.shadow = shadow != 0;
      chartModified = true;
   }

   private final int getShadow()
   {
      return shadow ? 1 : 0;
   }

   // ------------------------------------------------------------------------
   // get/set margin
   // ------------------------------------------------------------------------

   private static final String MARGIN_ATTR_NAME = "margin";
   private int margin = 0;

   private final void setMargin(int margin)
   {
      if (margin < 0)
         margin = 0;
      else if (margin > 100)
         margin = 100;

      this.margin = margin;
      attributeModified = true;
   }

   private final int getMargin()
   {
      return margin;
   }

   // ------------------------------------------------------------------------
   // get/set stacked
   // ------------------------------------------------------------------------

   private static final String STACKED_ATTR_NAME = "stacked";
   private boolean stacked = false;

   private final void setStacked(int stacked)
   {
      this.stacked = stacked != 0;
      chartModified = true;
   }

   private final int getStacked()
   {
      return stacked ? 1 : 0;
   }

   // ------------------------------------------------------------------------
   // attributes
   // ------------------------------------------------------------------------

   private static final int PAINT3D_ATTR_ID = 1;
   private static final int OUTLINE_ATTR_ID = 2;
   private static final int SHADOW_ATTR_ID = 3;
   private static final int MARGIN_ATTR_ID = 4;
   private static final int STACKED_ATTR_ID = 5;

   public void onClone(Plugin clone)
   {
      super.onClone(clone);

      Bar cloneObj = (Bar)clone;
      cloneObj.setPaint3d(getPaint3d());
      cloneObj.setOutline(getOutline());
      cloneObj.setShadow(getShadow());
      cloneObj.setMargin(getMargin());
      cloneObj.setStacked(getStacked());
   }

   public boolean onSet(String attribute, java.util.List index,
                        UnifiedData data)
   {
      Integer attrInt = (Integer)attributeList.get(attribute);
      if (attrInt != null)
      {
         int indexElements = index != null ? index.size() : 0;
         switch (attrInt.intValue())
         {
            case PAINT3D_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setPaint3d(data.getIntData());
               break;

            case OUTLINE_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setOutline(data.getIntData());
               break;

            case SHADOW_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setShadow(data.getIntData());
               break;

            case MARGIN_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setMargin(data.getIntData());
               break;

            case STACKED_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setStacked(data.getIntData());
               break;
         }

         return true;
      }

      return super.onSet(attribute, index, data);
   }

   public UnifiedData onGet(String attribute, java.util.List index)
   {
      Integer attrInt = (Integer)attributeList.get(attribute);
      if (attrInt != null)
      {
         int indexElements = index != null ? index.size() : 0;
         switch (attrInt.intValue())
         {
            case PAINT3D_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getPaint3d());

            case OUTLINE_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getOutline());

            case SHADOW_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getShadow());

            case MARGIN_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getMargin());

            case STACKED_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getStacked());
         }
      }

      return super.onGet(attribute, index);
   }

   private static HashMap attributeList = new HashMap();
   static
   {
      attributeList.put(PAINT3D_ATTR_NAME, new Integer(PAINT3D_ATTR_ID));
      attributeList.put(OUTLINE_ATTR_NAME, new Integer(OUTLINE_ATTR_ID));
      attributeList.put(SHADOW_ATTR_NAME, new Integer(SHADOW_ATTR_ID));
      attributeList.put(MARGIN_ATTR_NAME, new Integer(MARGIN_ATTR_ID));
      attributeList.put(STACKED_ATTR_NAME, new Integer(STACKED_ATTR_ID));
   }
}
